
      DOUBLE PRECISION FUNCTION Estart(Iflag,Icalc,IR,Molinit,Einit,T)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2007 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c		Selects initial energy for trial
c
c	Iflag	= 0	Initialize Pstart
c		= 1	select Estart from Pstart
c	Icalc	= type of initial distribution
c		= 1	Monoenergetic at energy Einit
c		= 2	Thermal with energy offset Einit
c		= 3	Chemical activation from 'product' #IR
c		= 4	Read Pstart from file 'multiwell.pstart'
c	IR	= index of 'product' from which chemical activation originates
c	Molinit	= index of initial molecule (well)
c	Einit	= initial energy (relative to ZPE of Molinit; units: cm-1)
c	T	= Temperature (vibrational)
c       Edels   = lowest energy (cm-1) of distribution (e.g. energy offset)
c       Hstart  = energy step-size (cm-1)
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
 
      INTEGER i , Iflag , ichan , Icalc , Molinit , IR , IE , IRUN , 
     &        itest , IA , Igg
 
      DOUBLE PRECISION AA , RX , E , T , RAN1 , DENVR , Einit , 
     &                 X , Y , Y1 , Y2 , Y3 , Term , Tlast , Etherm ,
     &                 RVR , CC, DOUBNTERP , Statest , Tiggp , Tiggm
 

      EXTERNAL Etherm , RVR
 
      SAVE 

      Statest = 0.1/Egrain1

      IF ( Iflag.EQ.0 ) THEN                    ! Initialize Pstart
 
         IF ( Icalc.EQ.1 ) THEN                 ! DELTA FUNCTION: No need for initialization
            RETURN

         ELSEIF ( Icalc.EQ.2 ) THEN                     ! THERMAL DISTRIBUTION
            Edels = Einit                               ! Offset

         ELSEIF ( Icalc.EQ.3 ) THEN                     ! CHEMICAL ACTIVATION
            itest = 0
            DO 40 i = 1 , Nchan(Molinit)                ! Find reaction index
               IF ( Jto(Molinit,i).EQ.IR ) THEN
                  ichan = i
                  itest = 1
               ENDIF
 40         CONTINUE
            IF ( itest.EQ.0 ) THEN
               WRITE (6,99001) Molinit , IR
               STOP
            ENDIF
            Edels = Eor(Molinit,ichan)                   ! Offset (incorporates centrifugal correction)
            Einit = Edels
 
         ELSEIF ( Icalc.EQ.4 ) THEN             ! READ NORMALIZED CUMULATIVE DISTRIBUTION FROM FILE
            OPEN (12,FILE='DensData/'//'multiwell.pstart',STATUS='OLD')
            READ (12,*) Jsize , Hstart , Edels     ! Edels and Hstart entered in cm-1
            IF ( Jsize .GT. Imax ) THEN
               write(*,*) '*** FATAL: Pstart array is too long ***'
               STOP
            ENDIF
            Einit = Edels
            DO 20 i = 1 , Jsize
               READ (12,*) AA , Pstart(i)       ! AA is dummy real variable
 20         CONTINUE
            CLOSE (12)
            RETURN
 
         ENDIF
 
         Hstart = 4.d+00*Etherm(Molinit,T)/Isize        ! Set step size
         Hstart = MAX( Egrain1 , Hstart )               ! Make sure step size not too small
         Jsize = 20.d+00*Etherm(Molinit,T)/Hstart       ! Set array length
         IF ( Jsize .GT. Imax ) Jsize = Imax

         Tlast = 0.0D+00
         DO 50 i = 1 , Jsize
            IF ( Icalc.EQ.3 ) THEN                         ! CHEMICAL ACTIVATION
               if(itun( Molinit , ichan ) .eq. 1 ) then    ! with tunneling
                 Einit = Edels - Emax1                     ! start at Emax1 below the barrier
                 IF ( Einit .LT. 0.0 ) Einit = 0.0
                 E = Einit + (i-1)*Hstart
                 CALL NTERP2((E-Edels),IE,X)
                 Y1 = TRate(Molinit,ichan,IE-1)
                 Y2 = TRate(Molinit,ichan,IE)
                 Y3 = TRate(Molinit,ichan,IE+1)
                 CALL QUDINT(Y1,Y2,Y3,X,Y)
                 Y = exp(Y)                                     ! Rate constant k(E) [stored as log]
                 Term = Y*DENVR(Molinit,E)*exp(-hor*E/T)
               else
                 E = Edels + (i-1)*Hstart                       ! Chemical activation without tunneling
                 Y = RVR( Molinit , ichan , E-Edels , IE )      ! Interpolated k(E)
                 Term = Y*DENVR(Molinit,E)*exp(-hor*E/T)
               end if
            ELSEIF ( Icalc.EQ.2 ) THEN                          ! THERMAL, WITH OR WITHOUT OFFSET
               E = (i-1)*Hstart
               Term = DENVR( Molinit , E )*exp(-hor*E/T)
            ENDIF
            IF ( Hstart .EQ. Egrain1 ) THEN
               Pstart(i) = Hstart*Term                     ! Constant within each grain at low energies
            ELSE
               Pstart(i) = 0.5D+00*Hstart*(Term+Tlast)     ! Trapezoidal rule for larger steps
            ENDIF
            Tlast=Term
 50      CONTINUE
 
         DO 100 i = 2 , Jsize
            Pstart(i) = Pstart(i) + Pstart(i-1)          ! Cumulative
 100     CONTINUE
 
         DO 150 i = 1 , Jsize
            Pstart(i) = Pstart(i)/Pstart(Jsize)          ! Normalize Cumulative distribution
 150     CONTINUE
 
c ---------------------------------------------------------------------------------
c ---------------------------------------------------------------------------------
c ---------------------------------------------------------------------------------

      ELSEIF ( Iflag.EQ.1 ) THEN                ! Select initial energy from Pstart
 
         IF ( Icalc.EQ.1 ) THEN
            Estart = Einit                              ! For delta function
            IF (Estart .LE. Emax1) THEN                 ! Align Estart with grains
                Igg = INT( Estart/Egrain1 + 0.5d+00 )
                Estart = Egrain1*Igg
                IF (Estart .LT. 0.0d+00) Estart = 0.0d+00
            ENDIF
            RETURN
         ELSE                                           ! For THERMAL, CHEMACT, etc.
 
            RX = RAN1(IDUM)
            i = 0
            IRUN = 1
            DO WHILE ( IRUN .EQ. 1 )
               i = i + 1
               IF ( Pstart(i) .GT. RX ) IRUN = 0
            ENDDO
 
            Nstart(i) = Nstart(i) + 1           ! Record selected distribution for posterity

            IF ( i .GT. 1 ) THEN
               IF ( Hstart .EQ. Egrain1 ) THEN                          ! for Egrain1 steps
                  Estart = Einit + Hstart*(i-1)
                ELSE
                  Estart = Einit + Hstart*( (i-1) +                     ! interpolate for other steps
     &               (RX - Pstart(i-1))/(Pstart(i) - Pstart(i-1)) )

                  Igg = NINT( Estart/Egrain1 )                    ! align with energy grains at low energy
                  Estart = Egrain1*Igg
                  IF ( DENVR(Molinit,Estart) .LT. Statest ) THEN  ! if a state is not present, move up or down one grain
                    Tiggp = DENVR(Molinit,Estart+Egrain1)
                    Tiggm = DENVR(Molinit,Estart-Egrain1)
                    IF ( Tiggp .GT. Tiggm ) THEN
                      Estart = Estart + Egrain1
                    ELSE
                      Estart = Estart - Egrain1
                    ENDIF
                  ENDIF
               ENDIF
             ELSE
                 Estart = Einit
            ENDIF

         ENDIF  ! Icalc
      ENDIF     ! Iflag

      IF ( Estart.LT.0.0 ) Estart = 0.0d+00

      IF(DOUBNTERP(Estart,Molinit,CNORM) .LT. 0.001) write(20,*) Estart       ! Statement for testing purposes

      RETURN
 
99001 FORMAT (/'Data file error: CHEMACT rxn does not exist:',I3,' to',
     &        I3/'****** Hit RETURN to terminate run *******')
      END
