 
      FUNCTION rivr( Mol , i , IE , numbdens )
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2005 John R. Barker
c
c John R. Barker
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c jrbarker@umich.edu
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
      IMPLICIT NONE

      INCLUDE 'declare1.inc'

      INTEGER          Mol , i , IE

      DOUBLE PRECISION EE , colls , rativr , numbdens , rivr , vfrq ,
     &                rmax

      DOUBLE PRECISION vivr , vave , civr , pivr , tivr
      INTEGER     iivr
      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            vave( MaxWell , MaxChan ) ,
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

c
c     vivr(Mol,i)   : characteristic reaction frequency (as in RRK theory) (cm-1)
c     vave(Mol,i)   : average molecular frequency (cm-1)
c     pivr(Mol,i)   : bimolecular collision rate constant for collision-induced IVR
c     tivr(Mol,i)   : IVR threshold energy (cm-1), measured from the  reaction threshold (Eor)
c     civr(Mol,i,j) : coefficients for 2nd order fit of 
c                     k(ivr) = civr(-,-,1) + civr(-,-,2)*EE + cirv(-,-,3)*EE*EE
c                     where EE = (E - Eor) is the Energy measured from the rxn threshold
c     iivr(Mol,i)   : flag to calculate IVR (iivr=1), or not (iivr=0)
c

      IF ( iivr(Mol , i) .EQ. 0 ) THEN              ! Neglect IVR

         rivr = 1.0d+00

       ELSEIF (iivr(Mol , i) .EQ. 1 ) THEN          ! Calculate IVR transmission coefficient

        rmax = 2.d+00*2.9979d+10*vave( Mol , i )    ! Upper limit to IVR rate

         IF ( IE.LE.imax1 ) THEN                    ! EE = (E - Eor) is the Energy measured from the rxn threshold
            EE = (IE-1)*Egrain1
           ELSE
            EE = (IE-imax1-1)*Egrain2
         ENDIF
c
c        Calculate IVR rate constant = f(EE)
c

         IF ( EE .LT. tivr( Mol , i ) ) THEN                          ! Energy must be .GE. IVR threshold energy
           rativr = 0.0d+00
         ELSE
           rativr = civr( Mol , i , 1 ) +                             ! IVR rate constant = f(EE) [second order fit]
     &        (civr( Mol , i , 2 ) + civr( Mol , i , 3 )*EE )*EE
           IF ( rativr .LT. 0.0d+00 ) rativr = 0.0d+00
           IF ( rativr .GT. rmax )    rativr = rmax                   ! upper limit corresponding to vave(Mol,i) cm-1
         ENDIF

         vfrq = vivr(Mol , i)*2.9979d+10                              ! Convert from cm-1 to frequency (s-1)
         colls = pivr( Mol , i )*numbdens                             ! Collision-induced IVR pseudo-first-order rate

         rivr = (rativr + colls) / ( rativr + colls + vfrq )          ! Wolynes-Leitner IVR transmission coefficient
      ENDIF

      IF(rivr.LT.1.0d-10) rivr = 1.0d-10           ! ensures that log(rivr) will not be NAN when rivr is very small

      RETURN
      END
