c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 Andrea Maranzana and John R. Barker
c
c Andrea Maranzana
c andrea.maranzana@unito.it
c Department of General and Organic Chemistry
c University of Torino
c Corso Massimo D'Azeglio, 48
c Torino  10125
c ITALY
c ++39-011-670-7637
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!     READ THERMO.DAT  AND CALC ZPE

      subroutine read_dat

      include 'declare.inc'
      Double Precision Mass(2), dum , amass , Hfxn, Tmax
      CHARACTER readpot*15
      CHARACTER formula*90
      CHARACTER keyword*4 , VROT*4
      DIMENSION keyword(2)
      INTEGER ki

      hindrance=0
      Gorin=0
      c=0
      a1=0

      OPEN (UNIT=4,STATUS='OLD',FILE='thermo.dat')                      ! General Data file

      READ (4,*) Eunits , Sunits
 
      IF ( Eunits.EQ.'KCAL' .OR. Eunits.EQ.'kcal' ) THEN
         Rgas = 8.314472D+00/4.184d+00
      ELSEIF ( Eunits.EQ.'KJOU' .OR. Eunits.EQ.'kjou' ) THEN
         Rgas = 8.314472D+00
      ELSEIF ( Eunits.EQ.'CM-1' .OR. Eunits.EQ.'cm-1' ) THEN
         Rgas = 0.6950356D+00
      ENDIF
 
      IF ( Sunits.EQ.'ATM' .OR. Sunits.EQ.'atm' ) THEN
         UNITS = 'atmosphere'
      ELSEIF ( Sunits.EQ.'BAR' .OR. Sunits.EQ.'bar' ) THEN
         UNITS = 'bar'
      ELSEIF ( Sunits.EQ.'MCC' .OR. Sunits.EQ.'mcc' ) THEN
         UNITS = 'molecule/cc'
      ENDIF
c
c	Number of temperatures (K)
c
      READ (4,*) Nt
      READ (4,*) (T(j),j=1,Nt)  ! Read list of temperatures

C......To find Emax (cm-1), the upper limit for eigenvalues of Hindered rotor 

	Tmax=T(1)
	DO j=2, Nt
		IF(T(j).GT.Tmax) Tmax=T(j)	! To find Tmax (maximum of Temperature) 
	ENDDO
	IF(Tmax.LT.3000.0d0) THEN
        	Tmax=3000.0d0
	ENDIF	
	Emax=10.0d0*(0.6950356D+00)*Tmax	! Emax = 10RT

C.......

      Hdiff = 0.0
      Nreac = 0
      Nprod = 0
      Nctst = 0
 
      READ (4,*) Ns
c
c	Number of species
c
      DO 100 i = 1 , Ns
         Warning(i)=0
         READ (4,*) REPROD(i) , MOLNAME(i) , DelH(i) 
         READ (4,*) Formula
           CALL ChemFormula(i,Formula)
          
c
c	reac/prod, name, Enthalpy(0 K), number of elements
c   empirical formula
c
         AMU(i) = 0.0d+00
         Hfxn298(i) = 0.0d+00
         DO ne = 1 , Nelement(i)                                    ! Empirical formula
            CALL element( ATYPE(i,ne) , amass , Hfxn , Sfxn)
            AMU(i) = AMU(i) + Natom(i,ne)*amass                     ! Atomic masses
            Hfxn298(i) = Hfxn298(i) + Natom(i,ne)*Hfxn*1000.0       ! [H(298.15) - H(0)]/Rgas (enthalpy function)
            Sfel(i) = Sfel(i) + Natom(i,ne)*Sfxn                    ! Sf/Rgas entropy of elements at 298.15 K
         END DO
         Hfxn298(i) = Hfxn298(i)*Rgas
         Sfel(i) = Sfel(i)*Rgas
c         
         READ (4,99114) TITLE1(i)
         READ (4,99114) TITLE2(i)
         READ (4,99114) TITLE3(i)
c
c   Three comment lines
c
c   symmetry number, optical isomers, number of electronic levels
         READ (4,*) Sym(i) , Sopt(i) , Nele(i) 
c
c   electronic energy levels and degeneracies
         READ (4,*) (Elev(i,ne) , gele(i,ne) , ne=1,Nele(i))
c
c   number of internal degrees of freedom (rots + vibs) to be read,
c      and two keywords
         READ (4,*) N(i) , keyword(1) , keyword(2)
c     KEYWORDS:
c          VHAR = 'HAR' for harmonic vibs, 
c       or VHAR = 'OBS' FOR observed (0-1 transitions)
c      and, for qro and rot types [NOT hindered rotor types]:
c          VROT = 'AMUA' for moment of inertia in units of amu*Ang^2
c       or      = 'GMCM' for moment of inertia units of gram*cm^2
c       or      = 'CM-1' for rotational constant in units of cm^-1
c       or      = 'MHZ' for rotational constant in units of MHz
c         
         DO ki = 1 , 2
           IF ( keyword(ki) .EQ. 'HAR' .OR. keyword(ki) .EQ. 'har' .OR.
     &          keyword(ki) .EQ. 'Har' ) THEN
             VHAR = 'HAR'
           ELSEIF ( keyword(ki) .EQ. 'OBS' .OR. keyword(ki) .EQ. 'obs'
     &         .OR. keyword(ki) .EQ. 'Obs' ) THEN
             VHAR = 'OBS'
           ELSEIF ( keyword(ki) .EQ. 'AMUA' .OR. 
     &              keyword(ki) .EQ. 'amua' )THEN
             VROT = 'AMUA'
           ELSEIF ( keyword(ki) .EQ. 'GMCM' .OR. 
     &              keyword(ki) .EQ. 'gmcm' )THEN
             VROT = 'GMCM'
           ELSEIF ( keyword(ki) .EQ. 'CM-1' .OR. 
     &              keyword(ki) .EQ. 'cm-1' )THEN
             VROT = 'CM-1'
           ELSEIF ( keyword(ki) .EQ. 'MHZ' .OR. 
     &              keyword(ki) .EQ. 'mhz' .OR. 
     &              keyword(ki) .EQ. 'MHz' .OR. 
     &              keyword(ki) .EQ. 'Mhz' )THEN
             VROT = 'MHZ'
           ELSE
             OPEN( 3, status='UNKNOWN', file='thermo.ERROR')
             write(3,*) 'FATAL: unknown keyword: ', keyword(ki)
             write(*,*) 'FATAL: unknown keyword: ', keyword(ki)
             STOP
           ENDIF
         END DO

         IF ( Nele(i) .LT. 1 ) then
           OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
           write(7,*) '*** FATAL: Requires at least 1 electronic level'
           write(*,*) '*** FATAL: Requires at least 1 electronic level'
           STOP '*** FATAL: Requires at least 1 electronic level ***'
         ELSEIF ( Nele(i) .gt. 10 ) then
           Warning(i)=1
           OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
           write(7,*) "**** WARNING: the maximum number of energy levels
     &is 10. ****"
           write(*,*) "**** WARNING: the maximum number of energy levels
     &is 10. ****"
           write(*,*)  Nele(i)-10, " levels ignored."
           Nele(i)=10
         ENDIF
c
c               Electronic level energies and degeneracies (for Nele levels); note that
c               lowest electronic energy level should be at Elev = 0 cm-1.
c
            Nflag = 0
         IF ( REPROD(i).EQ.REAC ) THEN
            a1=a1+1
            mass(a1)=AMU(i)

            Nreac = Nreac + 1
            Hdiff = Hdiff - 1000.*DelH(i)       ! Hdiff in cal/mole or J/mole
            Nflag = 1
         ELSEIF ( REPROD(i).EQ.PROD ) THEN
            Nprod = Nprod + 1
            Hdiff = Hdiff + 1000.*DelH(i)       ! Hdiff in cal/mole or J/mole
            Nflag = 1
         ELSEIF ( REPROD(i).EQ.CTST) THEN
            Nctst = Nctst + 1
            Hdiff = Hdiff + 1000.*DelH(i)       ! Hdiff in cal/mole or J/mole
            Nflag = 1
         ELSEIF ( REPROD(i).EQ.NON ) THEN
            Nnone = Nnone + 1
         ELSE 
            OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
            write(7,*) '***FATAL: unrecognized reac/prod type: ',
     &          REPROD(i)
            stop 
         ENDIF

      IF ( Nctst .GT. 1 ) THEN
         OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
         write(7,*) 'Too many transition states; only is one allowed!'
         write(*,*) 'Too many transition states; only is one allowed!'
         STOP 'Too many transition states; only is one allowed!'
      ENDIF

      IF ( Nctst .EQ. 1 .AND. Nprod.GT.0 ) THEN
         OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
         write(7,*) '"prod" not allowed in Canonical TST calculation!'
         write(*,*) '"prod" not allowed in Canonical TST calculation!'
         STOP '"prod" not allowed in Canonical TST calculation!'
      ENDIF

 
         zzpe(i) = 0.0       
         IF ( N(i).GT.0 ) THEN
            DO 20 II = 1 , N(i)
               READ (4,*) MODE(i,II) , IDOF(i,II) , WE(i,II) , ANH(i,II)
     &                    , NG(i,II)

               IF ( WE(i,II) .LT. 0.0 ) THEN
                  OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
                  WRITE(*,*) '***FATAL: WE(i) negative value: ',WE(i,II)
                  WRITE(7,*) '***FATAL: WE(i) negative value: ',WE(i,II)
                  STOP
               ENDIF
     
              IF ( IDOF(i,II).EQ.VIB ) THEN                    ! VIBRATION
                if(NG(i,II).eq.10.AND.REPROD(i).EQ.PROD) then
                   freq=WE(i,II)
                   NG(i,II)=1
                endif
                  W(i,II) = WE(i,II)
                  IF ( VHAR .EQ. HAR ) THEN
                   WOBS(i,II) = WE(i,II)+2.*ANH(i,II)           ! WOBS = observed freq
                  ELSEIF ( VHAR .EQ. OBS ) THEN
                   WOBS(i,II) = WE(i,II)
                   WE(i,II)= WE(i,II)-2.*ANH(i,II)        ! Convert WE to harmonic frequency
                  ENDIF
                 qq=Qvib(10.d+00,WE(i,II),ANH(i,II),NG(i,II),Cvib,Svib,
     &                    Hvib,zap)
                  zzpe(i) = zzpe(i) + zap
               ELSEIF ( IDOF(i,II).EQ.BOX ) THEN                    ! PARTICLE IN A BOX 
                  W(i,II) = WE(i,II)
                  IF ( VHAR .EQ. HAR ) THEN
                   WOBS(i,II) = 3.0d+00*WE(i,II)                ! WOBS = observed freq (note that ANH is a placeholder: not used)
                  ELSEIF ( VHAR .EQ. OBS ) THEN
                   WOBS(i,II) = WE(i,II)
                   WE(i,II)= WE(i,II)/3.0d+00             ! Convert WE to harmonic frequency
                  ENDIF
                 zap = W(i,II)
                 zzpe(i) = zzpe(i) + zap
                ELSEIF ( IDOF(i,II).EQ.ROT ) THEN                ! CLASSICAL ROTOR
                  call rotunits( VROT , WE(i,II) )                ! unit conversion
                  AMOM(i,II) = WE(i,II)
                ELSEIF ( IDOF(i,II).EQ.QRO ) THEN                ! QUANTUM ROTOR
                  call rotunits( VROT , WE(i,II) )                ! unit conversion
                  AMOM(i,II) = WE(i,II)
             
                ELSEIF (IDOF(i,II).EQ. GOR) THEN                 ! GORIN
                  call rotunits( VROT , WE(i,II) )                ! unit conversion
                  AMOM(i,II) = WE(i,II)
                  Gorin=Gorin+1
                ELSEIF ( IDOF(i,II).EQ.FIT) THEN                ! HINDRANCE
                  call rotunits( VROT , WE(i,II) )                ! unit conversion
                  AMOM(i,II) = WE(i,II)
                  hindrance=hindrance+1
                  Weorig(hindrance)= WE(i,II)
                ELSEIF ( IDOF(i,II).EQ.TOP ) THEN               ! Symmetric Top
                  call rotunits( VROT , WE(i,II) )                ! unit conversion
                  AMOM(i,II) = WE(i,II)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

                ELSEIF ( IDOF(i,II).EQ.HRA ) THEN                ! SYMMETRICAL HINDERED ROTOR 
                  IF ( NG(i,II) .LT. 0 ) THEN
                      READ(4,*) NSIG(i,II)
                      NG(i,II) = -NG(i,II)
                    ELSE
                      NSIG(i,II) = NG(i,II)
                  ENDIF
                  W(i,II) = WE(i,II)                    ! small vibration frequency
                  AMOM(i,II) = ANH(i,II)                ! moment of inertia
                  call rotunits( VROT , AMOM(i,II) )                ! unit conversion
                  B(i,II) = 16.85763D+00/AMOM(i,II)     ! rotational constant (cm-1)
                  VV(i,II) = ((W(i,II)/NG(i,II))**2)/B(i,II)
                                                        ! hindrance barrier (cm-1)
C.............To find eigenvalues and zero-point energy

                NSV(i,II)=NSIG(i,II)

		CALL SHRLEV(Emax,EVh,B(i,II),VV(i,II),
     &		NSIG(i,II),1,IMAX(i,II),zap,'Vhrd1','Bhrd1',0.0d0,0.0d0)

		DO LL=1, IMAX(i,II)
			EV(i,II,LL)=EVh(LL)
		ENDDO

                  zzpe(i) = zzpe(i) + zap
                  IDOF(i,II) = HIN

CCCCCCCCCCCCCCCCCCCCCCCCCC

                ELSEIF ( IDOF(i,II).EQ.HRB ) THEN                ! SYMMETRICAL HINDERED ROTOR 
                  IF ( NG(i,II) .LT. 0 ) THEN
                      READ(4,*) NSIG(i,II)
                      NG(i,II) = -NG(i,II)
                    ELSE
                      NSIG(i,II) = NG(i,II)
                  ENDIF
                  W(i,II) = WE(i,II)            ! small vibration frequency
                  VV(i,II) = ANH(i,II)                  ! hindrance barrier (cm-1)
                  IF (VV(i,II) .LT. 1.0d-05) STOP
     &            'Rotation barrier too small. Hit RETURN to terminate'
                  B(i,II) = ((W(i,II)/NG(i,II))**2)/VV(i,II)
                                                        ! rotational constant (cm-1)
                  AMOM(i,II) = 16.85763D+00/B(i,II)     ! rotational constant (cm-1)

C.............To find eigenvalues and zero-point energy

                NSV(i,II)=NSIG(i,II)

                CALL SHRLEV(Emax,EVh,B(i,II),VV(i,II),
     &          NSIG(i,II),1,IMAX(i,II),zap,'Vhrd1','Bhrd1',0.0d0,0.0d0)


                DO LL=1, IMAX(i,II)
                        EV(i,II,LL)=EVh(LL)
                ENDDO

                  zzpe(i) = zzpe(i) + zap
                  IDOF(i,II) = HIN

CCCCCCCCCCCCCCCCCCCCCCCCCC

                ELSEIF ( IDOF(i,II).EQ.HRC ) THEN                ! SYMMETRICAL HINDERED ROTOR 
                  IF ( NG(i,II) .LT. 0) THEN
                      READ(4,*) NSIG(i,II)
                      NG(i,II) = -NG(i,II)
                    ELSE
                      NSIG(i,II) = NG(i,II)
                  ENDIF
                  AMOM(i,II) = WE(i,II)                 ! moment of inertia
                  call rotunits( VROT , AMOM(i,II) )                ! unit conversion
                  B(i,II) = 16.85763D+00/AMOM(i,II)     ! rotational constant (cm-1)
                  VV(i,II) = ANH(i,II)                  ! hindrance barrier (cm-1)
                  W(i,II) = NG(i,II)*SQRT(B(i,II)*VV(i,II))
                                                        ! small vibration frequency

C.............To find eigenvalues and zero-point energy

		NSV(i,II)=NSIG(i,II)

                CALL SHRLEV(Emax,EVh,B(i,II),VV(i,II),
     &          NSIG(i,II),1,IMAX(i,II),zap,'Vhrd1','Bhrd1',0.0d0,0.0d0)


                DO LL=1, IMAX(i,II)
                        EV(i,II,LL)=EVh(LL)
                ENDDO

                  zzpe(i) = zzpe(i) + zap
                  IDOF(i,II) = HIN

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

		ELSEIF ( IDOF(i,II).EQ.HRD ) THEN	 ! NON-RIGID, UNSYMMETRICAL HINDERED ROTOR

			NSIG(i,II) = NG(i,II)
			NCV(i,II)=INT(WE(i,II))
			NCB(i,II)=INT(ANH(i,II))
			READ(4,*) VHRD(i,II), NSV(i,II), Phav(i,II), 
     &				(CV(i,II,LL), LL=1, NCV(i,II))
			READ(4,*) BHRD(i,II), NSB(i,II), Phab(i,II), 
     &				(CB(i,II,LL), LL=1, NCB(i,II))

			DO LL=1, NCB(i,II)
				CBB(LL)=CB(i,II,LL)
			ENDDO
			DO LL=1, NCV(i,II)
				CVV(LL)=CV(i,II,LL)
			ENDDO 

C.............To find eigenvalues and zero-point energy

			CALL UHRLEV(Emax,EVh,NCB(i,II),NCV(i,II),
     &		CBB,CVV,NSV(i,II),NSB(i,II),IMAX(i,II),zap,
     &		VHRD(i,II),BHRD(i,II),Phav(i,II),Phab(i,II))
	                DO LL=1, IMAX(i,II)
        	                EV(i,II,LL)=EVh(LL)
                	ENDDO
			zzpe(i) = zzpe(i) + zap
			IDOF(i,II) = HRD

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

                ELSE
                  OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
                  write(7,*) "**** FATAL: DOF type not recognized: ",
     &                   IDOF(i,II)        
                  write(*,*) "**** FATAL: DOF type not recognized: ",
     &                   IDOF(i,II)        
                  stop
               ENDIF
 20         CONTINUE
!           IF(vts.ne.0.AND.REPROD(i).eq.PROD) read(4,*) re, DDelH
            mu=(mass(1)*mass(2))/(mass(1)+mass(2))
            beta=0.1217788*freq*SQRT(mu/De)
         ENDIF
 
 100  CONTINUE
  
      IF(hindrance.gt.0) READ (4,*) (Kexp(j),j=1,Nt)
      IF(Gorin.gt.0) then
         read(4,*) readpot   ! Potential
         read(4,*) freq      ! CM-1
         read(4,*) De        ! Eunit    Without ZPE
!        read(4,*) I2d       ! amu A**2
         read(4,*) re        ! Re, between centers of mass, A
         Deorig=De
         IF ( Eunits.EQ.'KCAL' .OR. Eunits.EQ.'kcal' ) THEN
          De=De*349.76
          ELSEIF ( Eunits.EQ.'KJOU' .OR. Eunits.EQ.'kjou' ) THEN
          De=De*83.59
         ENDIF

         a1=0
         do i=1,Ns
           IF(REPROD(i).EQ.REAC) then
            a1=a1+1
            mass(a1)=AMU(i)
           ENDIF 
         enddo 
         IF(a1.ne.2) then
           OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
           write(7,*) "**** FATAL: Wrong number of reactants ****"
           write(*,*) "**** FATAL: Wrong number of reactants ****"
           stop
         ENDIF

         mu=(mass(1)*mass(2))/(mass(1)+mass(2))   
!        re=SQRT(I2d/mu)
         beta=0.1217788*freq*SQRT(mu/De)
        
         readpot=readpot(1:lnblnk(readpot))
         IF(readpot.eq."MORSE".OR.readpot.eq."morse") then
          POT="MORSE"
         elseif(readpot.eq."VARSHNI".OR.readpot.eq."varshni") then
          POT="VARSHNI"
          beta=0.5*(beta-(1/re))/re
         elseif (readpot.eq."sMORSE".OR.readpot.eq."smorse") then
          POT="sMORSE"
          read(4,*) c        ! read stiff Morse Parameter
         ELSE
          OPEN( 7, status='UNKNOWN', file='thermo.ERROR')
          write(7,*) "**** FATAL: Potential not recognized"
          write(*,*) "**** FATAL: Potential not recognized"
          stop
         ENDIF

      ENDIF
        
      CLOSE(4)

99114 FORMAT (A120)
 
      end subroutine 
